/************************************************************************
Demo Code for Designing_Visual_Editors_in_Qt Article

Copyright (C) 2004-2007  Prashanth N Udupa, (prashanth @ vcreatelogic . com)
        VCreate Logic Private Limited, Bangalore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License Version 2 for more details.

You should have received a copy of the GNU General Public License
Version 2 along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA
************************************************************************/

#ifndef C_WRAPPER_CLASS_LAYER_H
#define C_WRAPPER_CLASS_LAYER_H

#include <QList>
#include <QObject>
#include <QGraphicsScene>
#include <QGraphicsRectItem>
#include <QGraphicsLineItem>

class vtkObject;
class vtkDataObject;
class vtkActor;
class vtkMapper;
class vtkRenderer;
class vtkRenderWindow;

// forward declarations
struct CWrapperLinkDesc;
struct CWrapperLinkData;
class CWrapper;
class CWrapperLink;
class CWrapperCanvas;

#define POLY_DATA_TYPE  101
#define MAPPER_TYPE     102
#define ACTOR_TYPE      103
#define RENDERER_TYPE   104
#define RWIN_TYPE       105

// Link descriptor
struct CWrapperLinkDesc
{
    CWrapperLinkDesc();

    QString linkName;
    int linkDataType;
    bool acceptMultipleInputs;
};

// Link Data
struct CWrapperLinkData
{
    CWrapperLinkData();

    CWrapperLinkDesc linkDesc;
    union {
        vtkDataObject* dataObject;
        vtkActor* actor;
        vtkMapper* mapper;
        vtkRenderer* renderer;
        vtkRenderWindow* renderWindow;
    } data;
};

// Base class for all problem objects in this project.
struct CWrapperData;
class CWrapper : public QObject, public QGraphicsRectItem
{
    Q_OBJECT
    Q_PROPERTY(QString Name READ name WRITE setName)

public:
    static bool link(CWrapper* output, int outLine, CWrapper* input, int inLine, QString* errorMsg=0);
    static bool unlink(CWrapper* output, int outLine, CWrapper* input, int inLine, QString* errorMsg=0);

    CWrapper();
    ~CWrapper();

    const QString name() const { return objectName(); }

    vtkObject* getVtkObject() const;

    virtual int inputLinkCount() const;
    virtual CWrapperLinkDesc inputLinkDesc(int index) const;
    virtual QRectF inputLinkRect(int index) const;
    QList<CWrapperLink*> inputLinksAt(int index) const;

    virtual int outputLinkCount() const;
    virtual CWrapperLinkDesc outputLinkDesc(int index) const;
    virtual QRectF outputLinkRect(int index) const;
    QList<CWrapperLink*> outputLinksAt(int index) const;

public slots:
    void setName(const QString name) { setObjectName(name); update(); }
    void removeInputLinks();
    void removeOutputLinks();
    void removeAllLinks();

protected:
    virtual bool hasInput(int index);
    virtual bool setInput(int index, const CWrapperLinkData & linkData);
    virtual bool removeInput(int index, const CWrapperLinkData & linkData);
    virtual bool getOutput(int index, CWrapperLinkData & output);
    virtual void setVtkObject(vtkObject* object);

protected: // inherited from QObject
    bool event(QEvent* event);

protected: // inherited from QGraphicsItem
    void paint(QPainter* p, const QStyleOptionGraphicsItem* opt, QWidget* w);
    QVariant itemChange(GraphicsItemChange change, const QVariant & value);

protected: // other stuff.
    void addLink(CWrapperLink* link);
    void removeLink(CWrapperLink* link);

private:
    friend class CWrapperLink;
    friend class CWrapperCanvas;
    CWrapperData* d;
};

// Link between two problem objects
struct CWrapperLinkData2;
class CWrapperLink : public QGraphicsLineItem
{
public:
    ~CWrapperLink();

    bool inDestructor() const;

    CWrapper* output() const;
    int outputLinkIndex() const;

    CWrapper* input() const;
    int inputLinkIndex() const;

protected:
    CWrapperLink(CWrapper* output, int outLine, CWrapper* input, int inLine);
    void updateLink();
    void setAutoUnlink(bool val);

private:
    friend class CWrapper;
    friend class CWrapperCanvas;
    CWrapperLinkData2* d;
};

// Canvas on which all the problem objects are placed
struct CWrapperCanvasData;
class CWrapperCanvas : public QGraphicsScene
{
    Q_OBJECT

public:
    static CWrapperCanvas & instance();
    ~CWrapperCanvas();

    int wrapperCount() const;
    CWrapper* wrapper(int index) const;
    CWrapper* wrapper(const QString name) const;

    int linkCount() const;
    CWrapperLink* link(int index) const;
    CWrapperLink* link(CWrapper* output, int outLine, CWrapper* input, int inLine);
    QList<CWrapperLink*> link(CWrapper* output, CWrapper* input);

    CWrapper* wrapperAt(const QPointF pos, int* inputLink=0, int* outputLink=0) const;

    void newCanvas();

signals:
    void wrapperSelected(QObject* obj);

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent* e);
    void mouseMoveEvent(QGraphicsSceneMouseEvent* e);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent* e);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent* e);
    void dragEnterEvent(QGraphicsSceneDragDropEvent* de);
    void dragMoveEvent(QGraphicsSceneDragDropEvent* de);
    void dropEvent(QGraphicsSceneDragDropEvent* de);

protected:
    CWrapperCanvas(QObject* parent=0);

    void addWrapper(CWrapper* wrapper);
    void removeWrapper(CWrapper* wrapper);

    void addLink(CWrapperLink* link);
    void removeLink(CWrapperLink* link);

private slots:
    void appAboutToQuit();

private:
    friend class CWrapper;
    friend class CWrapperLink;
    CWrapperCanvasData* d;
};

#endif

