/************************************************************************
Demo Code for Designing_Visual_Editors_in_Qt Article

Copyright (C) 2004-2007  Prashanth N Udupa, (prashanth @ vcreatelogic . com)
        VCreate Logic Private Limited, Bangalore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License Version 2 for more details.

You should have received a copy of the GNU General Public License
Version 2 along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA
************************************************************************/

#include "CWrapperClassSelectionList.h"
#include "CWrapperFactory.h"

#include <QTreeWidget>
#include <QMimeData>
#include <QHeaderView>
#include <QMouseEvent>
#include <QDataStream>
#include <QByteArray>
#include <QApplication>

struct CWrapperClassSelectionListData
{
    QPoint dragStartPosition;
};

CWrapperClassSelectionList::CWrapperClassSelectionList(QWidget* parent)
:QTreeWidget(parent)
{
    d = new CWrapperClassSelectionListData;

    setColumnCount(1);
    // header()->hide();

    CWrapperFactory & factory = CWrapperFactory::instance();
    for(int i=0; i<factory.count(); i++)
    {
        QTreeWidgetItem* item = new QTreeWidgetItem(this);
        item->setText(0, factory.classNameAt(i));
        item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
    }

    QFont f(font());
    f.setPointSizeF(f.pointSizeF() * 1.2f);
    setFont(f);

    setAlternatingRowColors(true);
}

CWrapperClassSelectionList::~CWrapperClassSelectionList()
{
    delete d;
}

void CWrapperClassSelectionList::mousePressEvent(QMouseEvent* me)
{
    d->dragStartPosition = me->pos();
    QTreeWidget::mousePressEvent(me);
}

void CWrapperClassSelectionList::mouseMoveEvent(QMouseEvent* me)
{
    if (!(me->buttons() & Qt::LeftButton))
        return;
    if ((me->pos() - d->dragStartPosition).manhattanLength() < QApplication::startDragDistance())
        return;

    QTreeWidgetItem* item = itemAt(d->dragStartPosition);
    if(item)
        new CWrapperClassSelectionListDragObject(this, item->text(0));
}


static QString mimeType = "VCreateLogic/WrapperClass";

bool CWrapperClassSelectionListDragObject::canDecode(const QMimeData* mimeData)
{
    if(mimeData->hasFormat(mimeType))
        return true;
    return false;
}

bool CWrapperClassSelectionListDragObject::decode(const QMimeData* mimeData, QString & className)
{
    QByteArray byteData = mimeData->data(mimeType);
    if(byteData.isEmpty())
        return false;

    QDataStream ds(&byteData, QIODevice::ReadOnly);
    ds >> className;
    return true;
}

CWrapperClassSelectionListDragObject::CWrapperClassSelectionListDragObject(QWidget* source, const QString className)
:QDrag(source)
{
    QByteArray byteData;
    {
        QDataStream ds(&byteData, QIODevice::WriteOnly);
        ds << className;
    }

    QMimeData *mimeData = new QMimeData;
    mimeData->setData(mimeType, byteData);
    mimeData->setData("text/plain", className.toAscii().data());
    setMimeData(mimeData);

    start(Qt::MoveAction);
}
