/************************************************************************
Demo Code for Designing_Visual_Editors_in_Qt Article

Copyright (C) 2004-2007  Prashanth N Udupa, (prashanth @ vcreatelogic . com)
        VCreate Logic Private Limited, Bangalore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License Version 2 for more details.

You should have received a copy of the GNU General Public License
Version 2 along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA
************************************************************************/

#include "CWrapperFactory.h"
#include "CWrapperClassLayer.h"

#include <QApplication>
#include <QList>

struct CWrapperFactoryData
{
    QList<IWrapperCreator*> wrapperCreators;
};

CWrapperFactory & CWrapperFactory::instance()
{
    static CWrapperFactory* theInstance = new CWrapperFactory(qApp);
    return *theInstance;
}

CWrapperFactory::CWrapperFactory(QObject* parent)
:QObject(parent)
{
    d = new CWrapperFactoryData;
    registerCreators();
}

CWrapperFactory::~CWrapperFactory()
{
    delete d;
}

int CWrapperFactory::count() const
{
    return d->wrapperCreators.count();
}

const QString CWrapperFactory::classNameAt(int index) const
{
    if(index < 0 || index >= d->wrapperCreators.count())
        return QString();
    return d->wrapperCreators[index]->className();
}

const QMetaObject* CWrapperFactory::metaObjectAt(int index) const
{
    if(index < 0 || index >= d->wrapperCreators.count())
        return 0;
    return d->wrapperCreators[index]->metaObject();
}

CWrapper* CWrapperFactory::createWrapper(const QString name) const
{
    for(int i=0; i<d->wrapperCreators.count(); i++)
    {
        IWrapperCreator* creator = d->wrapperCreators[i];
        if(creator->className() == name)
            return creator->createInstance();
    }
    return 0;
}

CWrapper* CWrapperFactory::createWrapper(int index) const
{
    if(index < 0 || index >= d->wrapperCreators.count())
        return 0;
    return d->wrapperCreators[index]->createInstance();
}

void CWrapperFactory::registerCreator(IWrapperCreator* creator)
{
    d->wrapperCreators.push_back(creator);
}

void CWrapperFactory::unregisterCreator(IWrapperCreator* creator)
{
    d->wrapperCreators.removeAll(creator);
}


