/************************************************************************
Demo Code for Designing_Visual_Editors_in_Qt Article

Copyright (C) 2004-2007  Prashanth N Udupa, (prashanth @ vcreatelogic . com)
        VCreate Logic Private Limited, Bangalore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License Version 2 for more details.

You should have received a copy of the GNU General Public License
Version 2 along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA
************************************************************************/

#ifndef C_WRAPPER_FACTORY_H
#define C_WRAPPER_FACTORY_H

#include <QObject>

class CWrapper;
class IWrapperCreator;

struct CWrapperFactoryData;
class CWrapperFactory : public QObject
{
    Q_OBJECT

public:
    static CWrapperFactory & instance();
    ~CWrapperFactory();

    int count() const;
    const QString classNameAt(int index) const;
    const QMetaObject* metaObjectAt(int index) const;
    CWrapper* createWrapper(const QString name) const;
    CWrapper* createWrapper(int index) const;

    void registerCreators();

protected:
    CWrapperFactory(QObject* parent=0);
    void registerCreator(IWrapperCreator* creator);
    void unregisterCreator(IWrapperCreator* creator);

private:
    CWrapperFactoryData* d;
};

class IWrapperCreator
{
public:
    virtual ~IWrapperCreator() { }
    virtual QString className() const = 0;
    virtual const QMetaObject* metaObject() const = 0;
    virtual CWrapper* createInstance() const = 0;
};

#define DECLARE_WRAPPER(Name, WrapperClass) \
class CWrapperCreatorFor_##Name : public IWrapperCreator \
{ \
    virtual QString className() const { return QString(#Name); } \
    virtual const QMetaObject* metaObject() const { return & WrapperClass::staticMetaObject; } \
    virtual CWrapper* createInstance() const { return new WrapperClass; } \
};

#define FACTORY_TABLE_BEGIN void CWrapperFactory::registerCreators() {
#define FACTORY_TABLE_ENTRY(Name) registerCreator(new CWrapperCreatorFor_##Name);
#define FACTORY_TABLE_END }

#endif
