/*=========================================================================
  This module is an extension of the "Visualization Toolkit 
  ( copyright (c) 1993-2000 Ken Martin, Will Schroeder, Bill Lorensen )".
  and combines it with "Qt (copyright (C) 1992-2000 Troll Tech AS)".
  =========================================================================
  vtkQt - copyright 2000 Matthias Koenig 
  koenig@isg.cs.uni-magdeburg.de
  http://wwwisg.cs.uni-magdeburg.de/~makoenig
  =========================================================================
  vtkQt Classes modified for VTK Editor Demo by Prashanth N Udupa
  VCreate Logic, Bangalore. EMail: prashanth @ vcreatelogic . com
  =========================================================================
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
  * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
   
  * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
   
  * Modified source versions must be plainly marked as such, and must not be
  misrepresented as being the original software.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  =========================================================================*/

// Qt headers always go first
#include <QCursor>
#include <QMouseEvent>
//Added by qt3to4:
#include <QCloseEvent>
#include <QFocusEvent>
#include <QKeyEvent>

#ifndef _WIN32
#include <QX11Info>
#endif

#include "CVtkQtRenderWindow.h"
#include "CVtkQtRenderWindowInteractor.h"
#include "vtkRendererCollection.h"
#include "vtkRenderer.h"
#include "vtkCellPicker.h"
#include "vtkProperty.h"

#define MAX_LIGHTS 8

//QApplication *CVtkQtRenderWindow::qtapp = NULL;

void CVtkQtRenderWindow::initApp()
{

}


CVtkQtRenderWindow *CVtkQtRenderWindow::New()
{
    initApp();
    return new CVtkQtRenderWindow(0);
}


CVtkQtRenderWindow::CVtkQtRenderWindow(const QGLFormat & format,
QWidget * parent, const QGLWidget * shareWidget)
:QGLWidget(format, parent, shareWidget)
{
    initCVtkQtRenderWindow();
}


CVtkQtRenderWindow::CVtkQtRenderWindow(QWidget * parent, const QGLWidget * shareWidget)
:QGLWidget(parent, shareWidget)
{
    initCVtkQtRenderWindow();
}


void CVtkQtRenderWindow::initCVtkQtRenderWindow()
{
    vtkDebugMacro(<<" CVtkQtRenderWindow constructor\n");
#ifdef _WIN32
    WindowId = QWidget::winId();
    DeviceContext = GetDC(WindowId);
    this->MultiSamples = 8;
    this->SetWindowName("vtkQt");
#else
    /*
    DisplayId = QPaintDevice::x11AppDisplay();
    ColorMap = QPaintDevice::x11AppColormap();
    WindowId = QPaintDevice::handle();
    */
    DisplayId = QX11Info::display();
    ColorMap = QX11Info::appColormap();
    WindowId = (Window)(winId());
#ifndef VTK_USE_MESA
    this->MultiSamples = GetGlobalMaximumNumberOfMultiSamples();
#endif
    if (this->WindowName)
        delete[]this->WindowName;
    this->WindowName =
        new char[strlen("vtkQt") + 1];
    strcpy(this->WindowName, "vtkQt");
    setWindowTitle(this->WindowName);
#endif
    SetSwapBuffers(false);
    SetSize(200, 200);
    SetPosition(0, 0);
    move(0, 0);
    this->Interactor = NULL;
    qtRenWinInt = NULL;
    setFocusPolicy(Qt::WheelFocus);

	pickedActor = 0;
	actorPicker = 0;
}


CVtkQtRenderWindow::~CVtkQtRenderWindow()
{
    short cur_light;
    vtkOpenGLRenderer *ren;

    makeCurrent();
    for (cur_light = GL_LIGHT0; cur_light < GL_LIGHT0 + MAX_LIGHTS;
        cur_light++)
    {
        glDisable((GLenum) cur_light);
    }

    // tell each of the renderers that this render window/graphics context
    // is being removed (the RendererCollection is removed by vtkRenderWindow's
    // destructor)
    this->Renderers->InitTraversal();
    for (ren = (vtkOpenGLRenderer *) this->Renderers->GetNextItemAsObject();
        ren != NULL;
        ren = (vtkOpenGLRenderer *) this->Renderers->GetNextItemAsObject())
    {
        ren->SetRenderWindow(NULL);
    }

    if(qtRenWinInt)
    {
        qtRenWinInt->Delete();
        qtRenWinInt = 0;
    }

    glFinish();
}


// Initialize the window for rendering.
void CVtkQtRenderWindow::initializeGL(void)
{
    vtkDebugMacro(<<" InitializeGL\n");

    vtkDebugMacro(<<" glMatrixMode ModelView\n");
    glMatrixMode(GL_MODELVIEW);

    vtkDebugMacro(<<" zbuffer enabled\n");
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);

    vtkDebugMacro(" texture stuff\n");
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

    // initialize blending for transparency
    vtkDebugMacro(<<" blend func stuff\n");
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);

    glEnable(GL_NORMALIZE);
    glAlphaFunc(GL_GREATER, 0);

    this->Mapped = 1;
}


void CVtkQtRenderWindow::resizeGL(int x, int y)
{
    vtkDebugMacro(<<" SetSize\n");
    if ((this->Size[0] != x) || (this->Size[1] != y))
    {
        this->Modified();
        this->Size[0] = x;
        this->Size[1] = y;
    }
}


void CVtkQtRenderWindow::PrintSelf(ostream & os, vtkIndent indent)
{
    this->vtkRenderWindow::PrintSelf(os, indent);
    os << indent << "MultiSamples: " << this->MultiSamples << "\n";
}


void CVtkQtRenderWindow::paintGL()
{
    vtkDebugMacro(<<" paintGL()\n");
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    vtkRenderWindow::Render();
    glFinish();
    QApplication::restoreOverrideCursor();
}


vtkRenderWindowInteractor *CVtkQtRenderWindow::MakeRenderWindowInteractor()
{
    qtRenWinInt = CVtkQtRenderWindowInteractor::New();
    this->Interactor = qtRenWinInt;
    qtRenWinInt->SetRenderWindow(this);
    return qtRenWinInt;
}


void CVtkQtRenderWindow::SetInteractor(CVtkQtRenderWindowInteractor * arenwin)
{
    qtRenWinInt = arenwin;
    vtkRenderWindow::SetInteractor(arenwin);
}


vtkActor* CVtkQtRenderWindow::GetPickedActor()
{
	return pickedActor;
}


void CVtkQtRenderWindow::mousePressEvent(QMouseEvent * me)
{
    if (qtRenWinInt)
        qtRenWinInt->mousePressEvent(me);

	me->accept();

	if(!actorPicker)
		actorPicker = vtkCellPicker::New();

	vtkRendererCollection* rCol = GetRenderers();
	if(rCol->GetNumberOfItems() == 0)
		return;

	double x = qtRenWinInt ? qtRenWinInt->GetEventPosition()[0] : double(me->pos().x());
	double y = qtRenWinInt ? qtRenWinInt->GetEventPosition()[1] : double(me->pos().y());

	vtkRenderer* ren = (vtkRenderer*)(rCol->GetItemAsObject(0));
	int success = actorPicker->Pick(x, y, 0.0f, ren);

	vtkActorCollection* aCol = actorPicker->GetActors();
	aCol->InitTraversal();
	vtkActor* newPickedActor = success != 0 ? aCol->GetNextActor() : 0;

	if(newPickedActor == pickedActor)
		return;

	if(pickedActor)
		emit actorDropped(pickedActor);

	pickedActor = newPickedActor;

	if(pickedActor)
		emit actorPicked(pickedActor);
}


void CVtkQtRenderWindow::mouseReleaseEvent(QMouseEvent * me)
{
    if (qtRenWinInt)
        qtRenWinInt->mouseReleaseEvent(me);
    me->accept();
}


void CVtkQtRenderWindow::mouseMoveEvent(QMouseEvent * me)
{
    if (qtRenWinInt)
        qtRenWinInt->mouseMoveEvent(me);
    me->accept();
}


void CVtkQtRenderWindow::keyPressEvent(QKeyEvent * ke)
{
    if (qtRenWinInt)
        qtRenWinInt->keyPressEvent(ke);
}


void CVtkQtRenderWindow::closeEvent(QCloseEvent* ce)
{
    emit renderWindowClosed();
    if (qtRenWinInt)
    {
        this->SetInteractor(0);
        qtRenWinInt = 0;
    }
    QGLWidget::closeEvent(ce);
}


void CVtkQtRenderWindow::SetSize(int w, int h)
{
    resizeGL(w, h);
    QGLWidget::resize(w, h);
}


void CVtkQtRenderWindow::SetSize(int a[2])
{
    this->SetSize(a[0], a[1]);
}


void CVtkQtRenderWindow::GetSize(int &w, int &h)
{
    w = Size[0];
    h = Size[1];
}


QSizePolicy CVtkQtRenderWindow::sizePolicy() const
{
    return QSizePolicy(QSizePolicy::MinimumExpanding,
        QSizePolicy::MinimumExpanding);
}


QSize CVtkQtRenderWindow::sizeHint() const
{
    return QSize(50, 50);
}


QSize CVtkQtRenderWindow::minimumSizeHint() const
{
    return QSize(50, 50);
}


QString CVtkQtRenderWindow::getActualClassName() const
{
#ifdef _WIN32
    return QString("vtkWin32OpenGLRenderWindow");
#else
#ifdef VTK_USE_MESA
    return QString("vtkXMesaRenderWindow");
#else
    return QString("vtkXOpenGLRenderWindow");
#endif
#endif
}


void CVtkQtRenderWindow::setFocusSignal(bool val)
{
    _EmitFocusSignal = val;
}


bool CVtkQtRenderWindow::focusSignal() const
{
    return _EmitFocusSignal;
}


void CVtkQtRenderWindow::refresh()
{
    if(vtkRenderWindow::GetRenderers())
    {
        vtkRenderer* ren =
            dynamic_cast<vtkRenderer*>(vtkRenderWindow::GetRenderers()->GetItemAsObject(0));
        if(ren) 
			ren->Modified();
    }
    QGLWidget::updateGL();
}


void CVtkQtRenderWindow::focusInEvent(QFocusEvent *)
{
    if(_EmitFocusSignal)
        emit gotFocus(this);
}


void CVtkQtRenderWindow::focusOutEvent(QFocusEvent *)
{

}


void CVtkQtRenderWindow::Render()
{
    updateGL();
}


void CVtkQtRenderWindow::MakeCurrent()
{
    QGLWidget::makeCurrent();
}


#ifndef _WIN32
int CVtkQtRenderWindow::GetDesiredDepth()
{
    return x11Info().depth();
}


Colormap CVtkQtRenderWindow::GetDesiredColormap()
{
    return (Colormap) x11Info().colormap();
}


Visual* CVtkQtRenderWindow::GetDesiredVisual()
{
    return (Visual *) x11Info().visual();
}
#endif
