/*=========================================================================
  This module is an extension of the "Visualization Toolkit 
  ( copyright (c) 1993-2000 Ken Martin, Will Schroeder, Bill Lorensen )".
  and combines it with "Qt (copyright (C) 1992-2000 Troll Tech AS)".
  =========================================================================
  vtkQt - copyright 2000 Matthias Koenig 
  koenig@isg.cs.uni-magdeburg.de
  http://wwwisg.cs.uni-magdeburg.de/~makoenig
  =========================================================================
  vtkQt Classes modified for VTK Editor Demo by Prashanth N Udupa
  VCreate Logic, Bangalore. EMail: prashanth @ vcreatelogic . com
  =========================================================================
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
  * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
   
  * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
   
  * Modified source versions must be plainly marked as such, and must not be
  misrepresented as being the original software.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  =========================================================================*/

#ifndef _CVtkQtRenderWindow_h
#define _CVtkQtRenderWindow_h

#include <QGLWidget>
#include <QApplication>
#include <QPaintDevice>
//Added by qt3to4:
#include <QMouseEvent>
#include <QKeyEvent>
#include <QCloseEvent>
#include <QFocusEvent>

#include "vtkToolkits.h"
#include "vtkOpenGLRenderer.h"
#include "vtkRendererCollection.h"
#ifdef _WIN32
#include "vtkWin32OpenGLRenderWindow.h"
#include "vtkWin32RenderWindowInteractor.h"
#else
#ifdef VTK_USE_MESA
#include "vtkXMesaRenderWindow.h"
#include "vtkMesaRenderer.h"
#else
#include "vtkXOpenGLRenderWindow.h"
#include "vtkOpenGLRenderer.h"
#endif
#endif

class CVtkQtRenderWindowInteractor;
class vtkActor;
class vtkActorCollection;
class vtkCellPicker;

#ifdef _WIN_32
#define VTK_QT_EXPORT __declspec (dllexport)
#else
#define VTK_QT_EXPORT
#endif

class VTK_QT_EXPORT CVtkQtRenderWindow: public QGLWidget
#ifndef Q_MOC_RUN
#ifdef _WIN32
, public vtkWin32OpenGLRenderWindow
#else
#ifdef VTK_USE_MESA
, public vtkXMesaRenderWindow
#else
, public vtkXOpenGLRenderWindow
#endif // VTK_USE_MESA
#endif // _WIN32
#endif // Q_MOC_RUN
{
    Q_OBJECT
    protected:
        int MultiSamples;
        long OldMonitorSetting;

    public:
        CVtkQtRenderWindow(QWidget * parent = 0, const QGLWidget * shareWidget = 0);
        CVtkQtRenderWindow(const QGLFormat & format, QWidget * parent = 0,
				const QGLWidget * shareWidget = 0);
        ~CVtkQtRenderWindow();
        static CVtkQtRenderWindow *New();
        static void initApp();
        void PrintSelf(ostream & os, vtkIndent indent);
        void Render();
        void MakeCurrent();
        void SetSize(int w, int h);
        void SetSize(int a[2]);
        void GetSize(int &w, int &h);
        vtkRenderWindowInteractor *MakeRenderWindowInteractor();
        void SetInteractor(CVtkQtRenderWindowInteractor *);

		vtkActor* GetPickedActor();

#ifdef _WIN32
        vtkTypeMacro(CVtkQtRenderWindow, vtkWin32OpenGLRenderWindow);
#else
#ifdef VTK_USE_MESA
        vtkTypeMacro(CVtkQtRenderWindow, vtkMesaRenderWindow);
#else
        vtkTypeMacro(CVtkQtRenderWindow, vtkXOpenGLRenderWindow);
#endif
#endif

#ifndef _WIN32
        int GetDesiredDepth();
        Colormap GetDesiredColormap();
        Visual *GetDesiredVisual();
#endif

        QSizePolicy sizePolicy() const;
        QSize sizeHint() const;
        QSize minimumSizeHint() const;

        QString getActualClassName() const;

        void setFocusSignal(bool val);
        bool focusSignal() const;

    public slots:
        void refresh();

    protected:
        void initializeGL();
        void paintGL();
        void resizeGL(int, int);

        void mousePressEvent(QMouseEvent *);
        void mouseReleaseEvent(QMouseEvent *);
        void mouseMoveEvent(QMouseEvent *);
        void keyPressEvent(QKeyEvent *);
        void focusInEvent(QFocusEvent *);
        void focusOutEvent(QFocusEvent *);

    protected:
        // Added for VTK Designer Project
        void closeEvent(QCloseEvent*);

    signals:
        void renderWindowClosed();
        void gotFocus(CVtkQtRenderWindow*);
		void actorPicked(vtkActor* actor);
		void actorDropped(vtkActor* actor);

    private:
        void initCVtkQtRenderWindow();
        CVtkQtRenderWindowInteractor *qtRenWinInt;
        static QApplication *qtapp;
        bool _EmitFocusSignal;
		vtkActor* pickedActor;
		vtkCellPicker* actorPicker;
};
#endif
