/*=========================================================================
  This module is an extension of the "Visualization Toolkit 
  ( copyright (c) 1993-2000 Ken Martin, Will Schroeder, Bill Lorensen )".
  and combines it with "Qt (copyright (C) 1992-2000 Troll Tech AS)".
  =========================================================================
  vtkQt - copyright 2000 Matthias Koenig 
  koenig@isg.cs.uni-magdeburg.de
  http://wwwisg.cs.uni-magdeburg.de/~makoenig
  =========================================================================
  vtkQt Classes modified for VTK Editor Demo by Prashanth N Udupa
  VCreate Logic, Bangalore. EMail: prashanth @ vcreatelogic . com
  =========================================================================
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
  * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
   
  * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
   
  * Modified source versions must be plainly marked as such, and must not be
  misrepresented as being the original software.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  =========================================================================*/

#include <QMouseEvent>
#include <QKeyEvent>

#include "CVtkQtRenderWindowInteractor.h"
#include <ctype.h>
#include "vtkInteractorStyleSwitch.h"

CVtkQtRenderWindowInteractor::CVtkQtRenderWindowInteractor()
{
    qtRenWin = NULL;
}


CVtkQtRenderWindowInteractor::~CVtkQtRenderWindowInteractor()
{
    if(qtRenWin)
    {
        qtRenWin->SetInteractor(0);
    }
}


void CVtkQtRenderWindowInteractor::Start()
{
    if (!this->qtRenWin)
    {
        vtkErrorMacro(<<"No CVtkQtRenderWindow defined!");
        return;
    }
    if (!qApp)
    {
        vtkErrorMacro(<<"No QApplication defined!");
        return;
    }

    /*if (qApp->mainWidget() == 0)
    {
        qApp->setMainWidget(qtRenWin);
        qApp->exec();
    }*/
}


void CVtkQtRenderWindowInteractor::SetRenderWindow(CVtkQtRenderWindow * aren)
{
    qtRenWin = aren;
    aren->SetInteractor(this);
    vtkRenderWindowInteractor::SetRenderWindow(aren);
    qtRenWin->GetSize(Size[0], Size[1]);
}


void CVtkQtRenderWindowInteractor::PrintSelf(ostream & os, vtkIndent indent)
{
    vtkRenderWindowInteractor::PrintSelf(os, indent);
}


void CVtkQtRenderWindowInteractor::mousePressEvent(QMouseEvent * me)
{
    if (!Enabled)
        return;

    qtRenWin->GetSize(Size[0], Size[1]);

    int ctrl = 0, shift = 0;
    if (me->modifiers() & Qt::ControlModifier)
        ctrl = 1;
    if (me->modifiers() & Qt::ShiftModifier)
        shift = 1;
    int xp = me->x();
    int yp = me->y();
    SetEventInformationFlipY(xp, yp, ctrl, shift);

    switch (me->button())
    {
        case Qt::LeftButton:
            InvokeEvent(vtkCommand::LeftButtonPressEvent, NULL);
            break;
        case Qt::MidButton:
            InvokeEvent(vtkCommand::MiddleButtonPressEvent, NULL);
            break;
        case Qt::RightButton:
            InvokeEvent(vtkCommand::RightButtonPressEvent, NULL);
            break;
        default:
            return;
    }
}


void CVtkQtRenderWindowInteractor::mouseReleaseEvent(QMouseEvent * me)
{
    if (!Enabled)
        return;

    qtRenWin->GetSize(Size[0], Size[1]);

    int ctrl = 0, shift = 0;
    if (me->modifiers() & Qt::ControlModifier)
        ctrl = 1;
    if (me->modifiers() & Qt::ShiftModifier)
        shift = 1;
    int xp = me->x();
    int yp = me->y();
    SetEventInformationFlipY(xp, yp, ctrl, shift);

    switch (me->button())
    {
        case Qt::LeftButton:
            InvokeEvent(vtkCommand::LeftButtonReleaseEvent, NULL);
            break;
        case Qt::MidButton:
            InvokeEvent(vtkCommand::MiddleButtonReleaseEvent, NULL);
            break;
        case Qt::RightButton:
            InvokeEvent(vtkCommand::RightButtonReleaseEvent, NULL);
            break;
        default:
            return;
    }

}


void CVtkQtRenderWindowInteractor::mouseMoveEvent(QMouseEvent * me)
{
    if (!Enabled)
        return;

    qtRenWin->GetSize(Size[0], Size[1]);

    int ctrl = 0, shift = 0;
    if (me->modifiers() & Qt::ControlModifier)
        ctrl = 1;
    if (me->modifiers() & Qt::ShiftModifier)
        shift = 1;
    int xp = me->x();
    int yp = me->y();
    SetEventInformationFlipY(xp, yp, ctrl, shift);
    InvokeEvent(vtkCommand::MouseMoveEvent, NULL);
}


void CVtkQtRenderWindowInteractor::timer()
{
    if (!Enabled)
        return;
    InvokeEvent(vtkCommand::TimerEvent, NULL);
}


int CVtkQtRenderWindowInteractor::CreateTimer(int timertype)
{
    if (timertype == VTKI_TIMER_FIRST)
    {
        QObject::connect(&qTimer, SIGNAL(timeout()), SLOT(timer()));
        qTimer.start(10);
    }
    return 1;
}


int CVtkQtRenderWindowInteractor::DestroyTimer()
{
    qTimer.stop();
    QObject::disconnect(&qTimer, SIGNAL(timeout()), this, 0);
    return 1;
}


void CVtkQtRenderWindowInteractor::keyPressEvent(QKeyEvent * ke)
{
    if (!Enabled)
        return;

    // Avoid processing exit
    if((ke->text() == "E") || (ke->text() == "e"))
        return;

    qtRenWin->GetSize(Size[0], Size[1]);
    int ctrl = 0, shift = 0;
    if (ke->modifiers() & Qt::ControlModifier)
        ctrl = 1;
    if (ke->modifiers() & Qt::ShiftModifier)
        shift = 1;
    QPoint cp = qtRenWin->mapFromGlobal(QCursor::pos());
    int xp = cp.x();
    int yp = cp.y();
    SetEventInformationFlipY(xp, yp, ctrl, shift,
        (char) tolower(ke->key()), 1,
        (const char *) ke->text().toAscii().data());
    InvokeEvent(vtkCommand::KeyPressEvent, NULL);
    InvokeEvent(vtkCommand::CharEvent, NULL);
}
