/************************************************************************
Demo Code for Designing_Visual_Editors_in_Qt Article

Copyright (C) 2004-2007  Prashanth N Udupa, (prashanth @ vcreatelogic . com)
        VCreate Logic Private Limited, Bangalore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License Version 2 for more details.

You should have received a copy of the GNU General Public License
Version 2 along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA
************************************************************************/

#include "CVtkActor.h"
#include "vtkActor.h"

CVtkActor::CVtkActor()
{
    _vtkActor = vtkActor::New();
    setVtkObject(_vtkActor);
}

CVtkActor::~CVtkActor()
{

}

void CVtkActor::setPosition(QList<QVariant> pos)
{
    if(pos.count() != 3)
        return;
    _vtkActor->SetPosition(pos[0].toDouble(), pos[1].toDouble(), pos[2].toDouble());
}

QList<QVariant> CVtkActor::position() const
{
    QList<QVariant> ret;
    ret << _vtkActor->GetPosition()[0] <<
           _vtkActor->GetPosition()[1] <<
           _vtkActor->GetPosition()[2];
    return ret;
}

void CVtkActor::setScale(QList<QVariant> scale)
{
    if(scale.count() != 3)
        return;
    _vtkActor->SetScale(scale[0].toDouble(), scale[1].toDouble(), scale[2].toDouble());
}

QList<QVariant> CVtkActor::scale() const
{
    QList<QVariant> ret;
    ret << _vtkActor->GetScale()[0] <<
           _vtkActor->GetScale()[1] <<
           _vtkActor->GetScale()[2];
    return ret;
}

int CVtkActor::inputLinkCount() const
{
    return 1;
}

CWrapperLinkDesc CVtkActor::inputLinkDesc(int index) const
{
    CWrapperLinkDesc desc;

    if(!index)
    {
        desc.linkName = "Mapper";
        desc.linkDataType = MAPPER_TYPE;
    }

    return desc;
}

int CVtkActor::outputLinkCount() const
{
    return 1;
}

CWrapperLinkDesc CVtkActor::outputLinkDesc(int index) const
{
    CWrapperLinkDesc desc;

    if(!index)
    {
        desc.linkName = "Actor";
        desc.linkDataType = ACTOR_TYPE;
    }

    return desc;
}

bool CVtkActor::hasInput(int index)
{
    if(index)
        return false;

    return _vtkActor->GetMapper() != 0;
}

bool CVtkActor::setInput(int index, const CWrapperLinkData & linkData)
{
    if(index)
        return false;

    _vtkActor->SetMapper(linkData.data.mapper);
    return true;
}

bool CVtkActor::removeInput(int index, const CWrapperLinkData & linkData)
{
    if(index)
        return false;

    if(linkData.data.mapper != _vtkActor->GetMapper())
        return false;

    _vtkActor->SetMapper(0);
    return true;
}

bool CVtkActor::getOutput(int index, CWrapperLinkData & output)
{
    if(index)
        return false;

    output.data.actor = _vtkActor;
    return true;
}


